@@ -5,6 +5,7 @@ from apimatic_core.types.datetime_format import DateTimeFormat
 from apimatic_core.utilities.api_helper import ApiHelper
 from apimatic_core.utilities.xml_helper import XmlHelper
 from tests.apimatic_core.base import Base
+from tests.apimatic_core.mocks.exceptions.api_exception import APIException
 from tests.apimatic_core.mocks.exceptions.global_test_exception import GlobalTestException
 from tests.apimatic_core.mocks.exceptions.local_test_exception import LocalTestException
 from tests.apimatic_core.mocks.exceptions.nested_model_exception import NestedModelException
@@ -23,7 +24,8 @@ class TestResponseHandler(Base):
     @pytest.mark.parametrize('http_response, expected_exception_type, expected_error_message', [
         (Base.response(status_code=400), GlobalTestException, '400 Global'),
         (Base.response(status_code=412), NestedModelException, 'Precondition Failed'),
-        (Base.response(status_code=429), GlobalTestException, 'Invalid response')
+        (Base.response(status_code=429), GlobalTestException, 'Invalid response'),
+        (Base.response(status_code=399), GlobalTestException, '3XX Global')
     ])
     def test_global_error(self, http_response, expected_exception_type, expected_error_message):
         with pytest.raises(expected_exception_type) as error:
@@ -36,6 +38,26 @@ class TestResponseHandler(Base):
                 .handle(self.response(status_code=404), self.global_errors())
         assert error.value.reason == 'Not Found'
 
+    def test_default_local_error(self):
+        with pytest.raises(LocalTestException) as error:
+            self.new_response_handler.local_error(404, 'Not Found', LocalTestException) \
+                .local_error('default', 'Response Not OK', LocalTestException) \
+                .handle(self.response(status_code=412), self.global_errors())
+        assert error.value.reason == 'Response Not OK'
+
+    @pytest.mark.parametrize('http_response, expected_exception_type, expected_error_message', [
+        (Base.response(status_code=501), APIException, '5XX local'),
+        (Base.response(status_code=443), LocalTestException, '4XX local'),
+        (Base.response(status_code=522), LocalTestException, '522 local')
+    ])
+    def test_default_range_local_error(self, http_response, expected_exception_type, expected_error_message):
+        with pytest.raises(expected_exception_type) as error:
+            self.new_response_handler.local_error(522, '522 local', LocalTestException) \
+                .local_error('5XX', '5XX local', APIException) \
+                .local_error('4XX', '4XX local', LocalTestException) \
+                .handle(http_response, self.global_errors())
+        assert error.value.reason == expected_error_message
+
     def test_local_error_with_body(self):
         with pytest.raises(LocalTestException) as error:
             self.new_response_handler.local_error(404, 'Not Found', LocalTestException) \
@@ -48,6 +70,23 @@ class TestResponseHandler(Base):
                and error.value.server_message == 'Test message from server' \
                and error.value.secret_message_for_endpoint == 'This is test error message'
 
+    def test_local_error_template_message(self):
+        with pytest.raises(LocalTestException) as error:
+            self.new_response_handler.local_error_template(404, 'error_code => {$statusCode}, '
+                                                                'header => {$response.header.accept}, '
+                                                                'body => {$response.body#/ServerCode} - '
+                                                                '{$response.body#/ServerMessage} - '
+                                                                '{$response.body#/SecretMessageForEndpoint}',
+                                                           LocalTestException) \
+                .handle(self.response(status_code=404, text='{"ServerCode": 5001, "ServerMessage": '
+                                                            '"Test message from server", "SecretMessageForEndpoint": '
+                                                            '"This is test error message"}',
+                                      headers={'accept': 'application/json'}),
+                        self.global_errors_with_template_message())
+        assert error.value.reason == 'error_code => 404, ' \
+                                     'header => application/json, ' \
+                                     'body => 5001 - Test message from server - This is test error message'
+
     def test_global_error_with_body(self):
         with pytest.raises(NestedModelException) as error:
             self.new_response_handler.local_error(404, 'Not Found', LocalTestException) \
@@ -67,6 +106,17 @@ class TestResponseHandler(Base):
                and error.value.model.name == 'Test name' \
                and error.value.model.address == 'Test address'
 
+    def test_global_error_template_message(self):
+        with pytest.raises(NestedModelException) as error:
+            self.new_response_handler.local_error(404, 'Not Found', LocalTestException) \
+                .handle(self.response(status_code=412,
+                                      text='{"ServerCode": 5001, "ServerMessage": "Test message from server", "model": '
+                                           '{ "field": "Test field", "name": "Test name", "address": "Test address"}}'),
+                        self.global_errors_with_template_message())
+
+        assert error.value.reason == 'global error message -> error_code => 412, header => ,' \
+                                     ' body => 5001 - Test message from server - Test name'
+
     def test_local_error_precedence(self):
         with pytest.raises(LocalTestException) as error:
             self.new_response_handler.local_error(400, '400 Local', LocalTestException) \
